/** 
 * Copyright (c) SpaceToad, 2011
 * http://www.mod-buildcraft.com
 * 
 * BuildCraft is distributed under the terms of the Minecraft Mod Public 
 * License 1.0, or MMPL. Please check the contents of the license located in
 * http://www.mod-buildcraft.com/MMPL-1.0.txt
 */

package net.minecraft.src.buildcraft.api;

import java.util.LinkedList;
import java.util.TreeMap;

import net.minecraft.src.ItemStack;
import net.minecraft.src.World;

/**
 * This class records a slot, either from a blueprint or from a block placed
 * in the world.
 */
public class BptSlot {
	
	public enum Mode {ClearIfInvalid, Build};
	
	public Mode mode = Mode.Build;
	
	public int blockId = 0;
	public int meta = 0;
	public int x;
	public int y;
	public int z;	
	
	/**
	 * This tree contains additional data to be stored in the blueprint, 
	 * in the form "name" => "value".
	 */
	public TreeMap <String, String> cpt = new TreeMap <String, String> ();

	public ItemStack stackToUse;
	
	public LinkedList <ItemStack> getRequirements () {
		LinkedList <ItemStack> res = new LinkedList<ItemStack>();

		if (BuildCraftAPI.blockBptProps [blockId] == null) {
			res.add(new ItemStack(blockId, 1, meta));
		} else {
			BuildCraftAPI.blockBptProps [blockId].addRequirements (this, res);
		}
		
		return res;
	}
	
	public final void buildBlock (World world) {
		if (BuildCraftAPI.blockBptProps [blockId] == null) {
			//  Meta needs to be specified twice, depending on the block behavior
			world.setBlockAndMetadataWithNotify(x, y, z, blockId, meta);
			world.setBlockMetadataWithNotify(x, y, z, meta);
		} else {
			BuildCraftAPI.blockBptProps [blockId].buildBlock (this, world);			
		}
		
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public BptSlot clone () {
		BptSlot obj = new BptSlot();
		
		obj.x = x;
		obj.y = y;
		obj.z = z;
		obj.blockId = blockId;
		obj.meta = meta; 
		obj.cpt = (TreeMap<String, String>) cpt.clone();
		
		if (stackToUse != null) {
			obj.stackToUse = stackToUse.copy();
		}
		
		return obj;
	}

	public boolean isValid(World world) {
		if (BuildCraftAPI.blockBptProps [blockId] == null) {		
			// by default, this is a permissive check, doesn't check for 
			// metadata.
			return this.blockId == world.getBlockId(x, y, z);
		} else {
			return BuildCraftAPI.blockBptProps [blockId].isValid (world, this);
		}
		
	}

	public void rotateLeft() {
		if (BuildCraftAPI.blockBptProps [blockId] != null) {			
			BuildCraftAPI.blockBptProps [blockId].rotateLeft (this);
		}
	}
	
	public boolean ignoreBuilding() {
		if (BuildCraftAPI.blockBptProps [blockId] == null) {		
			return false;
		} else {
			return BuildCraftAPI.blockBptProps [blockId].ignoreBuilding(this);
		}
		
	}

	public void initializeFromWorld(World worldObj, int xs, int ys, int zs) {
		if (BuildCraftAPI.blockBptProps [blockId] != null) {		
			BuildCraftAPI.blockBptProps [blockId].initializeFromWorld(this, worldObj, xs, ys, zs);
		}
	}

	public void postProcessing(World world) {
		if (BuildCraftAPI.blockBptProps [blockId] != null) {		
			BuildCraftAPI.blockBptProps [blockId].postProcessing (this, world);
		}
	}

}